import cherrypy
from cherrypy.lib.filter import basefilter
import string

class AuthConfigError(Exception):
    """
    A problem with the AuthFilter has occurred.
    """
    pass

class AccessDeniedError(Exception):
    """
    The user was denied access.
    """
    pass

_USER_KEY = '_user'

def get_user():
    """
    Get the user object, None if user isn't logged in.
    """
    
    return cherrypy.session.get(_USER_KEY, None)
        

def login_user(user):
    """
    Specify a user has logged in.
    
    The parameter is a user object. This can be anything,
    such as a custom object (an "avatar") containing
    user information, or just a string containing the username.
    """
    
    cherrypy.session[_USER_KEY] = user


def logout_user():
    """
    Specify a user has logged out.
    """    
    
    if _USER_KEY in cherrypy.session:        
        del cherrypy.session[_USER_KEY]


# -----------------------------------------------------------------------------

MESSAGE_NOT_LOGGED_IN = "You are not logged in."

MESSAGE_DOT_NOT_HAVE_ROLE = "You do not have the requested roles."
#@@ MESSAGE_DOT_NOT_HAVE_ROLE = "You do not have the $role roles."

class AuthFilter(basefilter.BaseFilter):
    
    def beforeMain(self):
        
        # is the filter turned on?
        if not cherrypy.config.get('subway.auth.on', False):
            return
        
        try:
            # first, make sure he's logged in.
            user = cherrypy.session.get(_USER_KEY, None)
            if not user:
                message = cherrypy.config.get("subway.auth.not_logged_in_message", MESSAGE_NOT_LOGGED_IN)            
                raise AccessDeniedError(message)
            
            # next, if there are required_roles, check them
            required_roles = cherrypy.config.get("subway.auth.required_roles", None)
            if required_roles:            
                # there are required roles, check them            
                try:
                    # turn into a list of strings
                    user_roles = [str(role) for role in user.roles]
                except AttributeError:
                    cherrypy.log('Login failed, user object did not have a \'roles\' attribute, and the requested resource required roles.', severity=2)
                    raise AuthConfigError("User object did not have a 'roles' attribute, and the requested resource required roles.")
                
                # intersect the two role lists and see what remains
                if set(user_roles) & set(required_roles):
                    # ok, user has at least one required role
                    pass
                else:
                    # we use a dict here, so you are free of not to provide 
                    #  a role placeholder in your message
                    cherrypy.log('Login failed, user did not have the requested roles.')
                    message = cherrypy.config.get("subway.auth.no_role_message", MESSAGE_DOT_NOT_HAVE_ROLE) % dict(role=required_roles)
                    raise AccessDeniedError(message)                                
        
        except AccessDeniedError, exc:
            redirect_url = cherrypy.config.get("subway.auth.login_form_url", None)   
            if redirect_url:
                # the login function may want to know what the message and referer URL were, 
                # so provide them via special variables        
                
                # we use string.Template here (requires Python 2.4)
                                
                #raise cherrypy.HTTPRedirect(redirect_url.replace("$url", quote(cherrypy.request.browserUrl)).replace("$message", str(e)))
                raise cherrypy.HTTPRedirect(string.Template(redirect_url).substitute(url=cherrypy.request.path, message=str(exc)))
            
            # if the user did not specify a redirect URL, just re-raise
            cherrypy.log('No redirect URL given, please check subway.auth.login_form_url', severity=1)
            raise exc
        


